@@ -2,15 +2,12 @@
 //!
 //! Those are raw definitions so for documentation see the official
 //! [kernel documentation](https://www.kernel.org/doc/html/latest/input/joydev/joystick-api.html#joystick-api).
-
 #![no_std]
 
 extern crate input_event_codes;
 extern crate libc;
 extern crate nix;
 
-use core::clone::Clone;
-use core::marker::Copy;
 use core::mem::size_of;
 
 use input_event_codes::{ABS_CNT, BTN_MISC, KEY_CNT};
@@ -27,7 +24,7 @@ pub const JS_EVENT_AXIS: __u8 = 0x02;
 pub const JS_EVENT_INIT: __u8 = 0x80;
 
 #[allow(non_camel_case_types)]
-#[derive(Copy, Debug)]
+#[cfg_attr(feature = "extra_traits", derive(Debug, Eq, Hash, PartialEq))]
 #[repr(C)]
 pub struct js_event {
 	pub time: __u32,
@@ -36,6 +33,8 @@ pub struct js_event {
 	pub number: __u8,
 }
 
+impl Copy for js_event {}
+
 impl Clone for js_event {
 	fn clone(&self) -> Self {
 		*self
@@ -63,7 +62,7 @@ pub const JS_CORR_NONE: __u16 = 0x00;
 pub const JS_CORR_BROKEN: __u16 = 0x01;
 
 #[allow(non_camel_case_types)]
-#[derive(Copy, Debug)]
+#[cfg_attr(feature = "extra_traits", derive(Debug, Eq, Hash, PartialEq))]
 #[repr(C)]
 pub struct js_corr {
 	pub coef: [__s32; 8],
@@ -71,6 +70,8 @@ pub struct js_corr {
 	pub type_: __u16,
 }
 
+impl Copy for js_corr {}
+
 impl Clone for js_corr {
 	fn clone(&self) -> Self {
 		*self
@@ -100,7 +101,7 @@ pub const JS_GET_ALL: __u8 = 7;
 pub const JS_SET_ALL: __u8 = 8;
 
 #[allow(non_camel_case_types)]
-#[derive(Copy, Debug)]
+#[cfg_attr(feature = "extra_traits", derive(Debug, Eq, Hash, PartialEq))]
 #[repr(C)]
 pub struct JS_DATA_TYPE {
 	pub buttons: __s32,
@@ -108,6 +109,8 @@ pub struct JS_DATA_TYPE {
 	pub y: __s32,
 }
 
+impl Copy for JS_DATA_TYPE {}
+
 impl Clone for JS_DATA_TYPE {
 	fn clone(&self) -> Self {
 		*self
@@ -116,7 +119,7 @@ impl Clone for JS_DATA_TYPE {
 
 #[allow(non_camel_case_types)]
 #[allow(non_snake_case)]
-#[derive(Copy, Debug)]
+#[cfg_attr(feature = "extra_traits", derive(Debug, Eq, Hash, PartialEq))]
 #[repr(C)]
 pub struct JS_DATA_SAVE_TYPE_32 {
 	pub JS_TIMEOUT: __s32,
@@ -127,6 +130,8 @@ pub struct JS_DATA_SAVE_TYPE_32 {
 	pub JS_CORR: JS_DATA_TYPE,
 }
 
+impl Copy for JS_DATA_SAVE_TYPE_32 {}
+
 impl Clone for JS_DATA_SAVE_TYPE_32 {
 	fn clone(&self) -> Self {
 		*self
@@ -135,7 +140,7 @@ impl Clone for JS_DATA_SAVE_TYPE_32 {
 
 #[allow(non_camel_case_types)]
 #[allow(non_snake_case)]
-#[derive(Copy, Debug)]
+#[cfg_attr(feature = "extra_traits", derive(Debug, Eq, Hash, PartialEq))]
 #[repr(C)]
 pub struct JS_DATA_SAVE_TYPE_64 {
 	pub JS_TIMEOUT: __s32,
@@ -146,6 +151,8 @@ pub struct JS_DATA_SAVE_TYPE_64 {
 	pub JS_CORR: JS_DATA_TYPE,
 }
 
+impl Copy for JS_DATA_SAVE_TYPE_64 {}
+
 impl Clone for JS_DATA_SAVE_TYPE_64 {
 	fn clone(&self) -> Self {
 		*self
